#pragma once
#include "TProcuraMelhorativa.h"

///////////////////////////////////////////////////////////////////////////////
//	CDelegacias class
///////////////////////////////////////////////////////////////////////////////
//	Author: Jose Coelho
//	Last revision: 2024-03-13
//	Description:
//    Implementa o problema da colocao de delegacias, xerifes e deputados
//    - mapa rectangular, cada casa  uma zona que contm populao
//    - posicionar delegacias com seus xerifes, e deputados nas delegacias
//    - cada delegacia tem um alcance das zonas que protege
//    - o alcance  dependente do nmero de deputados
//		Deputados:
//			0 - raio 1 (apenas o xerife na delegacia) 
//			1 - raio 2
//			5 - raio 3
//			13 - raio 4
//    - os custos mensais de uma delegacia e o seu xerife  de 4 unidades
//    - cada deputado custa 1 unidade
//    - com um oramento limitado, posicionar delegacias e deputados
//    - a soluo  apenas vlida se um nmero mnimo de populao ficar protegido
///////////////////////////////////////////////////////////////////////////////

#define MAX_LARGURA 100
#define MAX_ALTURA 100

typedef struct {
	int linha, coluna; // coordenadas de uma zona
} TZona;

typedef struct {
	TZona zona; // posio da delegacia
	int alcance; // raio mximo de ao, com base nos deputados
} TDelegacia;

class CDelegacias :
	public TProcuraConstrutiva
{
private:
	// auxiliar
	static int protegido[MAX_ALTURA][MAX_LARGURA]; // 1 - casa protegida 0 - desprotegida
	int AtualizarProtecao(); // chamar aps alteraes no estado (ou antes de imprimir)
	int protegidos; // populao protegida pela soluo atual

	void Barra(int linha, int coluna);

	int TesteDeputados(); 
	// > considerar que as delegacias esto fixas, 
	//   e que todas recebem o mximo de delegados com a verba disponvel
	// > caso mesmo assim no seja possvel proteger a populao requerida, cortar

	int TesteAlcancaOutraDelegacia(int i); // verifica se a delegacia i alcana outra delegacia

public:
	CDelegacias(void);
	~CDelegacias(void);

	// informao esttica do problema
	static const int delegaciaAlcance[];

	// dados da instncia (estticos)
	static int largura, altura; // dimenses da zona
	static int populacao[MAX_ALTURA][MAX_LARGURA]; // populao (caregado uma s vez para a instncia)
	static int orcamento; // verba total mensal, que permite suportar delegacias e seus xerifes e deputados
	static int proteger; // valor mnimo da populao a proteger

	static void CarregarInstancia(int id, int versao);



	// soluo
	int disponivel; // verba disponvel do oramento
	TVector<TDelegacia> delegacia;
	// decises: construir delegacia, local, distribuir deputados

	// metodos redefinidos de TProcuraConstrutiva

	// Cria um objecto que e uma copia deste
	TProcuraConstrutiva* Duplicar(void);
	void Copiar(TProcuraConstrutiva* objecto);

	// Coloca o objecto no estado inicial da procura
	void SolucaoVazia(void);

	// Coloca em sucessores a lista de objectos sucessores (sao alocados neste metodo e tem de ser apagados)
	// O custo nao necessita de ser preenchido, caso seja sempre unitario
	void Sucessores(TVector<TProcuraConstrutiva*>&sucessores, TVector<int>&custo);
	// Retorna verdade caso o estado actual seja um estado objectivo
	bool SolucaoCompleta(void);
	// Escrever informacao de debug sobre o objecto currente
	// (utilizar variavel TProcuraConstrutiva::debug para seleccionar o detalhe pretendido)
	void Debug(void);
};
